# Copyright 2015 Gentoo Foundation
# Distributed under the terms of the GNU General Public License v2

from portage.tests import TestCase
from portage.tests.resolver.ResolverPlayground import (
    ResolverPlayground,
    ResolverPlaygroundTestCase,
)


class SonameUnsatisfiableTestCase(TestCase):
    def testSonameUnsatisfiable(self):

        binpkgs = {
            "app-misc/A-1": {
                "EAPI": "5",
                "PROVIDES": "x86_32: libA.so.1",
            },
            "app-misc/B-1": {
                "DEPEND": "app-misc/A",
                "RDEPEND": "app-misc/A",
                "REQUIRES": "x86_32: libA.so.2",
            },
            "app-misc/B-0": {
                "DEPEND": "app-misc/A",
                "RDEPEND": "app-misc/A",
                "REQUIRES": "x86_32: libA.so.1",
            },
        }

        installed = {
            "app-misc/A-1": {
                "EAPI": "5",
                "PROVIDES": "x86_32: libA.so.1",
            },
            "app-misc/B-0": {
                "DEPEND": "app-misc/A",
                "RDEPEND": "app-misc/A",
                "REQUIRES": "x86_32: libA.so.1",
            },
        }

        world = ["app-misc/B"]

        test_cases = (
            # Skip update due to unsatisfied soname dependency.
            ResolverPlaygroundTestCase(
                ["@world"],
                options={
                    "--deep": True,
                    "--ignore-soname-deps": "n",
                    "--update": True,
                    "--usepkgonly": True,
                },
                success=True,
                mergelist=[],
            ),
        )

        playground = ResolverPlayground(
            binpkgs=binpkgs, debug=False, installed=installed, world=world
        )
        try:
            for test_case in test_cases:
                playground.run_TestCase(test_case)
                self.assertEqual(test_case.test_success, True, test_case.fail_msg)
        finally:
            # Disable debug so that cleanup works.
            playground.debug = False
            playground.cleanup()



# Copyright 2017 Gentoo Foundation
# Distributed under the terms of the GNU General Public License v2

from portage.tests import TestCase
from portage.tests.resolver.ResolverPlayground import (
    ResolverPlayground,
    ResolverPlaygroundTestCase,
)


class BinaryPkgEbuildVisibilityTestCase(TestCase):
    def testBinaryPkgEbuildVisibility(self):

        binpkgs = {
            "app-misc/foo-3": {},
            "app-misc/foo-2": {},
            "app-misc/foo-1": {},
        }

        ebuilds = {
            "app-misc/foo-2": {},
            "app-misc/foo-1": {},
        }

        installed = {
            "app-misc/foo-1": {},
        }

        world = ["app-misc/foo"]

        test_cases = (
            # Test bug #612960, where --use-ebuild-visibility failed
            # to reject binary packages for which ebuilds were not
            # available.
            ResolverPlaygroundTestCase(
                ["@world"],
                options={
                    "--update": True,
                    "--deep": True,
                    "--use-ebuild-visibility": "y",
                    "--usepkgonly": True,
                },
                success=True,
                mergelist=[
                    "[binary]app-misc/foo-2",
                ],
            ),
            ResolverPlaygroundTestCase(
                ["@world"],
                options={
                    "--update": True,
                    "--deep": True,
                    "--usepkgonly": True,
                },
                success=True,
                mergelist=[
                    "[binary]app-misc/foo-3",
                ],
            ),
            ResolverPlaygroundTestCase(
                ["@world"],
                options={
                    "--update": True,
                    "--deep": True,
                    "--usepkg": True,
                },
                success=True,
                mergelist=[
                    "[binary]app-misc/foo-2",
                ],
            ),
            ResolverPlaygroundTestCase(
                ["=app-misc/foo-3"],
                options={
                    "--use-ebuild-visibility": "y",
                    "--usepkgonly": True,
                },
                success=False,
            ),
            ResolverPlaygroundTestCase(
                ["app-misc/foo"],
                options={
                    "--use-ebuild-visibility": "y",
                    "--usepkgonly": True,
                },
                success=True,
                mergelist=[
                    "[binary]app-misc/foo-2",
                ],
            ),
            ResolverPlaygroundTestCase(
                ["app-misc/foo"],
                options={
                    "--usepkgonly": True,
                },
                success=True,
                mergelist=[
                    "[binary]app-misc/foo-3",
                ],
            ),
            # The default behavior is to enforce ebuild visibility as
            # long as a visible package is available to satisfy the
            # current atom. In the following test case, ebuild visibility
            # is ignored in order to satisfy the =app-misc/foo-3 atom.
            ResolverPlaygroundTestCase(
                ["=app-misc/foo-3"],
                options={
                    "--usepkg": True,
                },
                success=True,
                mergelist=[
                    "[binary]app-misc/foo-3",
                ],
            ),
            # Verify that --use-ebuild-visibility works with --usepkg
            # when no other visible package is available.
            ResolverPlaygroundTestCase(
                ["=app-misc/foo-3"],
                options={
                    "--use-ebuild-visibility": "y",
                    "--usepkg": True,
                },
                success=False,
            ),
        )

        playground = ResolverPlayground(
            binpkgs=binpkgs, ebuilds=ebuilds, installed=installed, world=world
        )
        try:
            for test_case in test_cases:
                playground.run_TestCase(test_case)
                self.assertEqual(test_case.test_success, True, test_case.fail_msg)
        finally:
            playground.cleanup()



# Copyright 2012-2019 Gentoo Authors
# Distributed under the terms of the GNU General Public License v2

import textwrap

import portage
from portage import os
from portage.tests import TestCase
from portage.tests.resolver.ResolverPlayground import ResolverPlayground
from portage.util import ensure_dirs
from portage._global_updates import _do_global_updates


class MoveSlotEntTestCase(TestCase):
    def testMoveSlotEnt(self):

        ebuilds = {
            "dev-libs/A-2::dont_apply_updates": {
                "EAPI": "5",
                "SLOT": "0/2.30",
            },
            "dev-libs/B-2::dont_apply_updates": {
                "SLOT": "0",
            },
            "dev-libs/C-2.1::dont_apply_updates": {
                "EAPI": "5",
                "SLOT": "0/2.1",
            },
        }

        installed = {
            "dev-libs/A-1::test_repo": {
                "EAPI": "5",
                "SLOT": "0/2.30",
            },
            "dev-libs/B-1::test_repo": {
                "SLOT": "0",
            },
            "dev-libs/C-1::test_repo": {
                "EAPI": "5",
                "SLOT": "0/1",
            },
        }

        binpkgs = {
            "dev-libs/A-1::test_repo": {
                "EAPI": "5",
                "SLOT": "0/2.30",
            },
            "dev-libs/A-2::dont_apply_updates": {
                "EAPI": "5",
                "SLOT": "0/2.30",
            },
            "dev-libs/B-1::test_repo": {
                "SLOT": "0",
            },
            "dev-libs/B-2::dont_apply_updates": {
                "SLOT": "0",
            },
            "dev-libs/C-1::test_repo": {
                "EAPI": "5",
                "SLOT": "0/1",
            },
            "dev-libs/C-2.1::dont_apply_updates": {
                "EAPI": "5",
                "SLOT": "0/2.1",
            },
        }

        updates = textwrap.dedent(
            """
			slotmove dev-libs/A 0 2
			slotmove dev-libs/B 0 1
			slotmove dev-libs/C 0 1
		"""
        )

        playground = ResolverPlayground(
            binpkgs=binpkgs, ebuilds=ebuilds, installed=installed
        )

        settings = playground.settings
        trees = playground.trees
        eroot = settings["EROOT"]
        test_repo_location = settings.repositories["test_repo"].location
        portdb = trees[eroot]["porttree"].dbapi
        vardb = trees[eroot]["vartree"].dbapi
        bindb = trees[eroot]["bintree"].dbapi

        updates_dir = os.path.join(test_repo_location, "profiles", "updates")

        try:
            ensure_dirs(updates_dir)
            with open(os.path.join(updates_dir, "1Q-2010"), "w") as f:
                f.write(updates)

            # Create an empty updates directory, so that this
            # repo doesn't inherit updates from the main repo.
            ensure_dirs(
                os.path.join(
                    portdb.getRepositoryPath("dont_apply_updates"),
                    "profiles",
                    "updates",
                )
            )

            global_noiselimit = portage.util.noiselimit
            portage.util.noiselimit = -2
            try:
                _do_global_updates(trees, {})
            finally:
                portage.util.noiselimit = global_noiselimit

            # Workaround for cache validation not working
            # correctly when filesystem has timestamp precision
            # of 1 second.
            vardb._clear_cache()

            # 0/2.30 -> 2/2.30
            self.assertEqual("2/2.30", vardb.aux_get("dev-libs/A-1", ["SLOT"])[0])
            self.assertEqual("2/2.30", bindb.aux_get("dev-libs/A-1", ["SLOT"])[0])

            # 0 -> 1
            self.assertEqual("1", vardb.aux_get("dev-libs/B-1", ["SLOT"])[0])
            self.assertEqual("1", bindb.aux_get("dev-libs/B-1", ["SLOT"])[0])

            # 0/1 -> 1 (equivalent to 1/1)
            self.assertEqual("1", vardb.aux_get("dev-libs/C-1", ["SLOT"])[0])
            self.assertEqual("1", bindb.aux_get("dev-libs/C-1", ["SLOT"])[0])

            # dont_apply_updates
            self.assertEqual("0/2.30", bindb.aux_get("dev-libs/A-2", ["SLOT"])[0])
            self.assertEqual("0", bindb.aux_get("dev-libs/B-2", ["SLOT"])[0])
            self.assertEqual("0/2.1", bindb.aux_get("dev-libs/C-2.1", ["SLOT"])[0])

        finally:
            playground.cleanup()



# Copyright 2015 Gentoo Foundation
# Distributed under the terms of the GNU General Public License v2

from portage.tests import TestCase
from portage.tests.resolver.ResolverPlayground import (
    ResolverPlayground,
    ResolverPlaygroundTestCase,
)


class SonameSkipUpdateTestCase(TestCase):
    def testSonameSkipUpdate(self):

        binpkgs = {
            "app-misc/A-1": {
                "RDEPEND": "dev-libs/B",
                "DEPEND": "dev-libs/B",
                "REQUIRES": "x86_32: libB.so.1",
            },
            "dev-libs/B-2": {
                "PROVIDES": "x86_32: libB.so.2",
            },
            "dev-libs/B-1": {
                "PROVIDES": "x86_32: libB.so.1",
            },
        }

        installed = {
            "app-misc/A-1": {
                "RDEPEND": "dev-libs/B",
                "DEPEND": "dev-libs/B",
                "REQUIRES": "x86_32: libB.so.1",
            },
            "dev-libs/B-1": {
                "PROVIDES": "x86_32: libB.so.1",
            },
        }

        world = ("app-misc/A",)

        test_cases = (
            # Test that --ignore-soname-deps allows the upgrade,
            # even though it will break an soname dependency of
            # app-misc/A-1.
            ResolverPlaygroundTestCase(
                ["@world"],
                options={
                    "--deep": True,
                    "--ignore-soname-deps": "y",
                    "--update": True,
                    "--usepkgonly": True,
                },
                success=True,
                mergelist=[
                    "[binary]dev-libs/B-2",
                ],
            ),
            # Test that upgrade to B-2 is skipped with --usepkgonly
            # because it will break an soname dependency that
            # cannot be satisfied by the available binary packages.
            ResolverPlaygroundTestCase(
                ["@world"],
                options={
                    "--deep": True,
                    "--ignore-soname-deps": "n",
                    "--update": True,
                    "--usepkgonly": True,
                },
                success=True,
                mergelist=[],
            ),
        )

        playground = ResolverPlayground(
            debug=False, binpkgs=binpkgs, installed=installed, world=world
        )
        try:
            for test_case in test_cases:
                playground.run_TestCase(test_case)
                self.assertEqual(test_case.test_success, True, test_case.fail_msg)
        finally:
            # Disable debug so that cleanup works.
            playground.debug = False
            playground.cleanup()



# Copyright 2012-2021 Gentoo Authors
# Distributed under the terms of the GNU General Public License v2

import textwrap

import portage
from portage import os
from portage.tests import TestCase
from portage.tests.resolver.ResolverPlayground import ResolverPlayground
from portage.util import ensure_dirs
from portage._global_updates import _do_global_updates


class MoveEntTestCase(TestCase):
    def testMoveEnt(self):

        ebuilds = {
            "dev-libs/A-2::dont_apply_updates": {
                "EAPI": "4",
                "SLOT": "2",
            },
        }

        installed = {
            "dev-libs/A-1::test_repo": {
                "EAPI": "4",
            },
            "dev-libs/A-2::dont_apply_updates": {
                "EAPI": "4",
                "SLOT": "2",
            },
        }

        binpkgs = {
            "dev-libs/A-1::test_repo": {
                "EAPI": "4",
            },
            "dev-libs/A-2::dont_apply_updates": {
                "EAPI": "4",
                "SLOT": "2",
            },
        }

        updates = textwrap.dedent(
            """
			move dev-libs/A dev-libs/A-moved
		"""
        )

        playground = ResolverPlayground(
            binpkgs=binpkgs, ebuilds=ebuilds, installed=installed
        )

        settings = playground.settings
        trees = playground.trees
        eroot = settings["EROOT"]
        test_repo_location = settings.repositories["test_repo"].location
        portdb = trees[eroot]["porttree"].dbapi
        vardb = trees[eroot]["vartree"].dbapi
        bindb = trees[eroot]["bintree"].dbapi

        updates_dir = os.path.join(test_repo_location, "profiles", "updates")

        try:
            ensure_dirs(updates_dir)
            with open(os.path.join(updates_dir, "1Q-2010"), "w") as f:
                f.write(updates)

            # Create an empty updates directory, so that this
            # repo doesn't inherit updates from the main repo.
            ensure_dirs(
                os.path.join(
                    portdb.getRepositoryPath("dont_apply_updates"),
                    "profiles",
                    "updates",
                )
            )

            global_noiselimit = portage.util.noiselimit
            portage.util.noiselimit = -2
            try:
                _do_global_updates(trees, {})
            finally:
                portage.util.noiselimit = global_noiselimit

            # Workaround for cache validation not working
            # correctly when filesystem has timestamp precision
            # of 1 second.
            vardb._clear_cache()

            # A -> A-moved
            self.assertRaises(KeyError, vardb.aux_get, "dev-libs/A-1", ["EAPI"])
            vardb.aux_get("dev-libs/A-moved-1", ["EAPI"])
            # The original package should still exist because a binary
            # package move is a copy on write operation.
            bindb.aux_get("dev-libs/A-1", ["EAPI"])
            bindb.aux_get("dev-libs/A-moved-1", ["EAPI"])

            # dont_apply_updates
            self.assertRaises(KeyError, vardb.aux_get, "dev-libs/A-moved-2", ["EAPI"])
            vardb.aux_get("dev-libs/A-2", ["EAPI"])
            self.assertRaises(KeyError, bindb.aux_get, "dev-libs/A-moved-2", ["EAPI"])
            bindb.aux_get("dev-libs/A-2", ["EAPI"])

        finally:
            playground.cleanup()



# Copyright 1999-2012 Gentoo Foundation
# Distributed under the terms of the GNU General Public License v2

from _emerge.CompositeTask import CompositeTask
from _emerge.EbuildPhase import EbuildPhase

import portage
from portage import os


class EbuildBinpkg(CompositeTask):
    """
    This assumes that src_install() has successfully completed.
    """

    __slots__ = ("pkg", "settings") + ("_binpkg_tmpfile", "_binpkg_info")

    def _start(self):
        pkg = self.pkg
        root_config = pkg.root_config
        bintree = root_config.trees["bintree"]
        binpkg_tmpfile = os.path.join(
            bintree.pkgdir, pkg.cpv + ".tbz2." + str(portage.getpid())
        )
        bintree._ensure_dir(os.path.dirname(binpkg_tmpfile))

        self._binpkg_tmpfile = binpkg_tmpfile
        self.settings["PORTAGE_BINPKG_TMPFILE"] = self._binpkg_tmpfile

        package_phase = EbuildPhase(
            background=self.background,
            phase="package",
            scheduler=self.scheduler,
            settings=self.settings,
        )

        self._start_task(package_phase, self._package_phase_exit)

    def _package_phase_exit(self, package_phase):

        self.settings.pop("PORTAGE_BINPKG_TMPFILE", None)
        if self._default_exit(package_phase) != os.EX_OK:
            try:
                os.unlink(self._binpkg_tmpfile)
            except OSError:
                pass
            self.wait()
            return

        pkg = self.pkg
        bintree = pkg.root_config.trees["bintree"]
        self._binpkg_info = bintree.inject(pkg.cpv, filename=self._binpkg_tmpfile)

        self._current_task = None
        self.returncode = os.EX_OK
        self.wait()

    def get_binpkg_info(self):
        return self._binpkg_info



# Copyright 2013 Gentoo Foundation
# Distributed under the terms of the GNU General Public License v2

from portage.tests import TestCase
from portage.tests.resolver.ResolverPlayground import (
    ResolverPlayground,
    ResolverPlaygroundTestCase,
)


class SlotChangeWithoutRevBumpTestCase(TestCase):
    def testSlotChangeWithoutRevBump(self):

        ebuilds = {
            "app-arch/libarchive-3.1.1": {"EAPI": "5", "SLOT": "0/13"},
            "app-arch/libarchive-3.0.4-r1": {"EAPI": "5", "SLOT": "0"},
            "kde-base/ark-4.10.0": {
                "EAPI": "5",
                "DEPEND": "app-arch/libarchive:=",
                "RDEPEND": "app-arch/libarchive:=",
            },
        }

        binpkgs = {
            "app-arch/libarchive-3.1.1": {"EAPI": "5", "SLOT": "0"},
        }

        installed = {
            "app-arch/libarchive-3.1.1": {"EAPI": "5", "SLOT": "0"},
            "kde-base/ark-4.10.0": {
                "EAPI": "5",
                "DEPEND": "app-arch/libarchive:0/0=",
                "RDEPEND": "app-arch/libarchive:0/0=",
            },
        }

        world = ["kde-base/ark"]

        test_cases = (
            # Demonstrate bug #456208, where a sub-slot change
            # without revbump needs to trigger a rebuild.
            ResolverPlaygroundTestCase(
                ["kde-base/ark"],
                options={"--oneshot": True, "--usepkg": True},
                success=True,
                mergelist=["app-arch/libarchive-3.1.1", "kde-base/ark-4.10.0"],
            ),
            ResolverPlaygroundTestCase(
                ["app-arch/libarchive"],
                options={"--noreplace": True, "--usepkg": True},
                success=True,
                mergelist=[],
            ),
            ResolverPlaygroundTestCase(
                ["app-arch/libarchive"],
                options={"--usepkg": True},
                success=True,
                mergelist=["[binary]app-arch/libarchive-3.1.1"],
            ),
            # Test --changed-slot
            ResolverPlaygroundTestCase(
                ["@world"],
                options={
                    "--changed-slot": True,
                    "--usepkg": True,
                    "--update": True,
                    "--deep": True,
                },
                success=True,
                mergelist=["app-arch/libarchive-3.1.1", "kde-base/ark-4.10.0"],
            ),
        )

        playground = ResolverPlayground(
            ebuilds=ebuilds,
            binpkgs=binpkgs,
            installed=installed,
            world=world,
            debug=False,
        )
        try:
            for test_case in test_cases:
                playground.run_TestCase(test_case)
                self.assertEqual(test_case.test_success, True, test_case.fail_msg)
        finally:
            playground.cleanup()



# Copyright 2012-2013 Gentoo Foundation
# Distributed under the terms of the GNU General Public License v2

import re
import textwrap

import portage
from portage import os
from portage.dep import Atom
from portage.tests import TestCase
from portage.tests.resolver.ResolverPlayground import ResolverPlayground
from portage.update import update_dbentry
from portage.util import ensure_dirs
from portage.versions import _pkg_str
from portage._global_updates import _do_global_updates


class UpdateDbentryTestCase(TestCase):
    def testUpdateDbentryTestCase(self):
        cases = (
            (
                ("move", Atom("dev-libs/A"), Atom("dev-libs/B")),
                "1",
                "  dev-libs/A:0  ",
                "  dev-libs/B:0  ",
            ),
            (
                ("move", Atom("dev-libs/A"), Atom("dev-libs/B")),
                "1",
                "  >=dev-libs/A-1:0  ",
                "  >=dev-libs/B-1:0  ",
            ),
            (
                ("move", Atom("dev-libs/A"), Atom("dev-libs/B")),
                "2",
                "  dev-libs/A[foo]  ",
                "  dev-libs/B[foo]  ",
            ),
            (
                ("move", Atom("dev-libs/A"), Atom("dev-libs/B")),
                "5",
                "  dev-libs/A:0/1=[foo]  ",
                "  dev-libs/B:0/1=[foo]  ",
            ),
            (
                ("move", Atom("dev-libs/A"), Atom("dev-libs/B")),
                "5",
                "  dev-libs/A:0/1[foo]  ",
                "  dev-libs/B:0/1[foo]  ",
            ),
            (
                ("move", Atom("dev-libs/A"), Atom("dev-libs/B")),
                "5",
                "  dev-libs/A:0/0[foo]  ",
                "  dev-libs/B:0/0[foo]  ",
            ),
            (
                ("move", Atom("dev-libs/A"), Atom("dev-libs/B")),
                "5",
                "  dev-libs/A:0=[foo]  ",
                "  dev-libs/B:0=[foo]  ",
            ),
            (
                ("slotmove", Atom("dev-libs/A"), "0", "1"),
                "1",
                "  dev-libs/A:0  ",
                "  dev-libs/A:1  ",
            ),
            (
                ("slotmove", Atom("dev-libs/A"), "0", "1"),
                "1",
                "  >=dev-libs/A-1:0  ",
                "  >=dev-libs/A-1:1  ",
            ),
            (
                ("slotmove", Atom("dev-libs/A"), "0", "1"),
                "5",
                "  dev-libs/A:0/1=[foo]  ",
                "  dev-libs/A:1/1=[foo]  ",
            ),
            (
                ("slotmove", Atom("dev-libs/A"), "0", "1"),
                "5",
                "  dev-libs/A:0/1[foo]  ",
                "  dev-libs/A:1/1[foo]  ",
            ),
            (
                ("slotmove", Atom("dev-libs/A"), "0", "1"),
                "5",
                "  dev-libs/A:0/0[foo]  ",
                "  dev-libs/A:1/1[foo]  ",
            ),
            (
                ("slotmove", Atom("dev-libs/A"), "0", "1"),
                "5",
                "  dev-libs/A:0=[foo]  ",
                "  dev-libs/A:1=[foo]  ",
            ),
        )
        for update_cmd, eapi, input_str, output_str in cases:
            result = update_dbentry(update_cmd, input_str, eapi=eapi)
            self.assertEqual(result, output_str)

    def testUpdateDbentryBlockerTestCase(self):
        """
        Avoid creating self-blockers for bug #367215.
        """
        cases = (
            (
                ("move", Atom("dev-libs/A"), Atom("dev-libs/B")),
                _pkg_str("dev-libs/B-1", eapi="1", slot="0"),
                "  !dev-libs/A  ",
                "  !dev-libs/A  ",
            ),
            (
                ("move", Atom("dev-libs/A"), Atom("dev-libs/B")),
                _pkg_str("dev-libs/C-1", eapi="1", slot="0"),
                "  !dev-libs/A  ",
                "  !dev-libs/B  ",
            ),
            (
                ("move", Atom("dev-libs/A"), Atom("dev-libs/B")),
                _pkg_str("dev-libs/B-1", eapi="1", slot="0"),
                "  !dev-libs/A:0  ",
                "  !dev-libs/A:0  ",
            ),
            (
                ("move", Atom("dev-libs/A"), Atom("dev-libs/B")),
                _pkg_str("dev-libs/C-1", eapi="1", slot="0"),
                "  !dev-libs/A:0  ",
                "  !dev-libs/B:0  ",
            ),
            (
                ("move", Atom("dev-libs/A"), Atom("dev-libs/B")),
                _pkg_str("dev-libs/C-1", eapi="1", slot="0"),
                "  !>=dev-libs/A-1:0  ",
                "  !>=dev-libs/B-1:0  ",
            ),
            (
                ("move", Atom("dev-libs/A"), Atom("dev-libs/B")),
                _pkg_str("dev-libs/B-1", eapi="1", slot="0"),
                "  !>=dev-libs/A-1:0  ",
                "  !>=dev-libs/A-1:0  ",
            ),
            (
                ("move", Atom("dev-libs/A"), Atom("dev-libs/B")),
                _pkg_str("dev-libs/C-1", eapi="1", slot="0"),
                "  !>=dev-libs/A-1  ",
                "  !>=dev-libs/B-1  ",
            ),
            (
                ("move", Atom("dev-libs/A"), Atom("dev-libs/B")),
                _pkg_str("dev-libs/B-1", eapi="1", slot="0"),
                "  !>=dev-libs/A-1  ",
                "  !>=dev-libs/A-1  ",
            ),
        )
        for update_cmd, parent, input_str, output_str in cases:
            result = update_dbentry(update_cmd, input_str, parent=parent)
            self.assertEqual(result, output_str)

    def testUpdateDbentryDbapiTestCase(self):

        ebuilds = {
            "dev-libs/A-2::dont_apply_updates": {
                "RDEPEND": "dev-libs/M dev-libs/N dev-libs/P",
                "EAPI": "4",
                "SLOT": "2",
            },
            "dev-libs/B-2::dont_apply_updates": {
                "RDEPEND": "dev-libs/M dev-libs/N dev-libs/P",
                "EAPI": "4",
                "SLOT": "2",
            },
        }

        installed = {
            "dev-libs/A-1::test_repo": {
                "RDEPEND": "dev-libs/M dev-libs/N dev-libs/P",
                "EAPI": "4",
            },
            "dev-libs/A-2::dont_apply_updates": {
                "RDEPEND": "dev-libs/M dev-libs/N dev-libs/P",
                "EAPI": "4",
                "SLOT": "2",
            },
            "dev-libs/B-1::test_repo": {
                "RDEPEND": "dev-libs/M dev-libs/N dev-libs/P",
                "EAPI": "4-python",
            },
            "dev-libs/M-1::test_repo": {
                "EAPI": "4",
            },
            "dev-libs/N-1::test_repo": {
                "EAPI": "4",
            },
            "dev-libs/N-2::test_repo": {
                "EAPI": "4-python",
            },
        }

        binpkgs = {
            "dev-libs/A-1::test_repo": {
                "RDEPEND": "dev-libs/M dev-libs/N dev-libs/P",
                "EAPI": "4",
            },
            "dev-libs/A-2::dont_apply_updates": {
                "RDEPEND": "dev-libs/M dev-libs/N dev-libs/P",
                "EAPI": "4",
                "SLOT": "2",
            },
            "dev-libs/B-1::test_repo": {
                "RDEPEND": "dev-libs/M dev-libs/N dev-libs/P",
                "EAPI": "4-python",
            },
        }

        world = ["dev-libs/M", "dev-libs/N"]

        updates = textwrap.dedent(
            """
			move dev-libs/M dev-libs/M-moved
			move dev-libs/N dev-libs/N.moved
		"""
        )

        playground = ResolverPlayground(
            binpkgs=binpkgs, ebuilds=ebuilds, installed=installed, world=world
        )

        settings = playground.settings
        trees = playground.trees
        eroot = settings["EROOT"]
        test_repo_location = settings.repositories["test_repo"].location
        portdb = trees[eroot]["porttree"].dbapi
        vardb = trees[eroot]["vartree"].dbapi
        bindb = trees[eroot]["bintree"].dbapi
        setconfig = trees[eroot]["root_config"].setconfig
        selected_set = setconfig.getSets()["selected"]

        updates_dir = os.path.join(test_repo_location, "profiles", "updates")

        try:
            ensure_dirs(updates_dir)
            with open(os.path.join(updates_dir, "1Q-2010"), "w") as f:
                f.write(updates)

            # Create an empty updates directory, so that this
            # repo doesn't inherit updates from the main repo.
            ensure_dirs(
                os.path.join(
                    portdb.getRepositoryPath("dont_apply_updates"),
                    "profiles",
                    "updates",
                )
            )

            global_noiselimit = portage.util.noiselimit
            portage.util.noiselimit = -2
            try:
                _do_global_updates(trees, {})
            finally:
                portage.util.noiselimit = global_noiselimit

            # Workaround for cache validation not working
            # correctly when filesystem has timestamp precision
            # of 1 second.
            vardb._clear_cache()

            # M -> M-moved
            old_pattern = re.compile(r"\bdev-libs/M(\s|$)")
            rdepend = vardb.aux_get("dev-libs/A-1", ["RDEPEND"])[0]
            self.assertTrue(old_pattern.search(rdepend) is None)
            self.assertTrue("dev-libs/M-moved" in rdepend)
            rdepend = bindb.aux_get("dev-libs/A-1", ["RDEPEND"])[0]
            self.assertTrue(old_pattern.search(rdepend) is None)
            self.assertTrue("dev-libs/M-moved" in rdepend)
            rdepend = vardb.aux_get("dev-libs/B-1", ["RDEPEND"])[0]
            self.assertTrue(old_pattern.search(rdepend) is None)
            self.assertTrue("dev-libs/M-moved" in rdepend)
            rdepend = vardb.aux_get("dev-libs/B-1", ["RDEPEND"])[0]
            self.assertTrue(old_pattern.search(rdepend) is None)
            self.assertTrue("dev-libs/M-moved" in rdepend)

            # EAPI 4-python/*-progress N -> N.moved
            rdepend = vardb.aux_get("dev-libs/B-1", ["RDEPEND"])[0]
            old_pattern = re.compile(r"\bdev-libs/N(\s|$)")
            self.assertTrue(old_pattern.search(rdepend) is None)
            self.assertTrue("dev-libs/N.moved" in rdepend)
            rdepend = bindb.aux_get("dev-libs/B-1", ["RDEPEND"])[0]
            self.assertTrue(old_pattern.search(rdepend) is None)
            self.assertTrue("dev-libs/N.moved" in rdepend)
            self.assertRaises(KeyError, vardb.aux_get, "dev-libs/N-2", ["EAPI"])
            vardb.aux_get("dev-libs/N.moved-2", ["RDEPEND"])[0]

            # EAPI 4 does not allow dots in package names for N -> N.moved
            rdepend = vardb.aux_get("dev-libs/A-1", ["RDEPEND"])[0]
            self.assertTrue("dev-libs/N" in rdepend)
            self.assertTrue("dev-libs/N.moved" not in rdepend)
            rdepend = bindb.aux_get("dev-libs/A-1", ["RDEPEND"])[0]
            self.assertTrue("dev-libs/N" in rdepend)
            self.assertTrue("dev-libs/N.moved" not in rdepend)
            vardb.aux_get("dev-libs/N-1", ["RDEPEND"])[0]
            self.assertRaises(KeyError, vardb.aux_get, "dev-libs/N.moved-1", ["EAPI"])

            # dont_apply_updates
            rdepend = vardb.aux_get("dev-libs/A-2", ["RDEPEND"])[0]
            self.assertTrue("dev-libs/M" in rdepend)
            self.assertTrue("dev-libs/M-moved" not in rdepend)
            rdepend = bindb.aux_get("dev-libs/A-2", ["RDEPEND"])[0]
            self.assertTrue("dev-libs/M" in rdepend)
            self.assertTrue("dev-libs/M-moved" not in rdepend)

            selected_set.load()
            self.assertTrue("dev-libs/M" not in selected_set)
            self.assertTrue("dev-libs/M-moved" in selected_set)
            self.assertTrue("dev-libs/N" not in selected_set)
            self.assertTrue("dev-libs/N.moved" in selected_set)

        finally:
            playground.cleanup()



# Copyright 2017 Gentoo Foundation
# Distributed under the terms of the GNU General Public License v2

from portage.tests import TestCase
from portage.tests.resolver.ResolverPlayground import (
    ResolverPlayground,
    ResolverPlaygroundTestCase,
)


class AutounmaskBinpkgUseTestCase(TestCase):
    def testAutounmaskBinpkgUse(self):
        ebuilds = {
            "dev-libs/A-1": {
                "EAPI": "6",
                "DEPEND": "dev-libs/B[foo]",
                "RDEPEND": "dev-libs/B[foo]",
            },
            "dev-libs/B-1": {
                "EAPI": "6",
                "IUSE": "foo",
            },
        }
        binpkgs = {
            "dev-libs/A-1": {
                "EAPI": "6",
                "DEPEND": "dev-libs/B[foo]",
                "RDEPEND": "dev-libs/B[foo]",
            },
            "dev-libs/B-1": {
                "EAPI": "6",
                "IUSE": "foo",
                "USE": "foo",
            },
        }
        installed = {}

        test_cases = (
            # Bug 619626: Test for unnecessary rebuild due
            # to rejection of binary packages that would
            # be acceptable after appplication of autounmask
            # USE changes.
            ResolverPlaygroundTestCase(
                ["dev-libs/A"],
                all_permutations=True,
                success=True,
                options={
                    "--usepkg": True,
                },
                mergelist=[
                    "[binary]dev-libs/B-1",
                    "[binary]dev-libs/A-1",
                ],
                use_changes={"dev-libs/B-1": {"foo": True}},
            ),
        )

        playground = ResolverPlayground(
            ebuilds=ebuilds, binpkgs=binpkgs, installed=installed, debug=False
        )
        try:
            for test_case in test_cases:
                playground.run_TestCase(test_case)
                self.assertEqual(test_case.test_success, True, test_case.fail_msg)
        finally:
            playground.debug = False
            playground.cleanup()



# Copyright 2015 Gentoo Foundation
# Distributed under the terms of the GNU General Public License v2

from portage.tests import TestCase
from portage.tests.resolver.ResolverPlayground import (
    ResolverPlayground,
    ResolverPlaygroundTestCase,
)


class RebuiltBinariesCase(TestCase):
    def testRebuiltBinaries(self):

        user_config = {
            "make.conf": ('FEATURES="binpkg-multi-instance"',),
        }

        binpkgs = (
            (
                "app-misc/A-1",
                {
                    "EAPI": "5",
                    "BUILD_ID": "1",
                    "BUILD_TIME": "1",
                },
            ),
            (
                "app-misc/A-1",
                {
                    "EAPI": "5",
                    "BUILD_ID": "2",
                    "BUILD_TIME": "2",
                },
            ),
            (
                "app-misc/A-1",
                {
                    "EAPI": "5",
                    "BUILD_ID": "3",
                    "BUILD_TIME": "3",
                },
            ),
            (
                "dev-libs/B-1",
                {
                    "EAPI": "5",
                    "BUILD_ID": "1",
                    "BUILD_TIME": "1",
                },
            ),
            (
                "dev-libs/B-1",
                {
                    "EAPI": "5",
                    "BUILD_ID": "2",
                    "BUILD_TIME": "2",
                },
            ),
            (
                "dev-libs/B-1",
                {
                    "EAPI": "5",
                    "BUILD_ID": "3",
                    "BUILD_TIME": "3",
                },
            ),
        )

        installed = {
            "app-misc/A-1": {
                "EAPI": "5",
                "BUILD_ID": "1",
                "BUILD_TIME": "1",
            },
            "dev-libs/B-1": {
                "EAPI": "5",
                "BUILD_ID": "2",
                "BUILD_TIME": "2",
            },
        }

        world = (
            "app-misc/A",
            "dev-libs/B",
        )

        test_cases = (
            ResolverPlaygroundTestCase(
                ["@world"],
                options={
                    "--deep": True,
                    "--rebuilt-binaries": True,
                    "--update": True,
                    "--usepkgonly": True,
                },
                success=True,
                ignore_mergelist_order=True,
                mergelist=["[binary]dev-libs/B-1-3", "[binary]app-misc/A-1-3"],
            ),
        )

        playground = ResolverPlayground(
            debug=False,
            binpkgs=binpkgs,
            installed=installed,
            user_config=user_config,
            world=world,
        )
        try:
            for test_case in test_cases:
                playground.run_TestCase(test_case)
                self.assertEqual(test_case.test_success, True, test_case.fail_msg)
        finally:
            # Disable debug so that cleanup works.
            # playground.debug = False
            playground.cleanup()



# Copyright 2015 Gentoo Foundation
# Distributed under the terms of the GNU General Public License v2

from portage.tests import TestCase
from portage.tests.resolver.ResolverPlayground import (
    ResolverPlayground,
    ResolverPlaygroundTestCase,
)


class SonameSlotConflictUpdateTestCase(TestCase):
    def testSonameSlotConflictUpdate(self):

        binpkgs = {
            "app-text/podofo-0.9.2": {
                "RDEPEND": "dev-util/boost-build",
            },
            "dev-cpp/libcmis-0.3.1": {
                "DEPEND": "dev-libs/boost",
                "RDEPEND": "dev-libs/boost",
                "REQUIRES": "x86_32: libboost-1.53.so",
            },
            "dev-libs/boost-1.53.0": {
                "PROVIDES": "x86_32: libboost-1.53.so",
                "RDEPEND": "=dev-util/boost-build-1.53.0",
            },
            "dev-libs/boost-1.52.0": {
                "PROVIDES": "x86_32: libboost-1.52.so",
                "RDEPEND": "=dev-util/boost-build-1.52.0",
            },
            "dev-util/boost-build-1.53.0": {},
            "dev-util/boost-build-1.52.0": {},
        }

        installed = {
            "app-text/podofo-0.9.2": {
                "RDEPEND": "dev-util/boost-build",
            },
            "dev-cpp/libcmis-0.3.1": {
                "DEPEND": "dev-libs/boost",
                "RDEPEND": "dev-libs/boost",
                "REQUIRES": "x86_32: libboost-1.52.so",
            },
            "dev-util/boost-build-1.52.0": {},
            "dev-libs/boost-1.52.0": {
                "PROVIDES": "x86_32: libboost-1.52.so",
                "RDEPEND": "=dev-util/boost-build-1.52.0",
            },
        }

        world = [
            "dev-cpp/libcmis",
            "dev-libs/boost",
            "app-text/podofo",
        ]

        test_cases = (
            ResolverPlaygroundTestCase(
                world,
                all_permutations=True,
                options={
                    "--deep": True,
                    "--ignore-soname-deps": "n",
                    "--update": True,
                    "--usepkgonly": True,
                },
                success=True,
                mergelist=[
                    "[binary]dev-util/boost-build-1.53.0",
                    "[binary]dev-libs/boost-1.53.0",
                    "[binary]dev-cpp/libcmis-0.3.1",
                ],
            ),
            ResolverPlaygroundTestCase(
                world,
                all_permutations=True,
                options={
                    "--deep": True,
                    "--ignore-soname-deps": "y",
                    "--update": True,
                    "--usepkgonly": True,
                },
                success=True,
                mergelist=[
                    "[binary]dev-util/boost-build-1.53.0",
                    "[binary]dev-libs/boost-1.53.0",
                ],
            ),
        )

        playground = ResolverPlayground(
            binpkgs=binpkgs, installed=installed, world=world, debug=False
        )
        try:
            for test_case in test_cases:
                playground.run_TestCase(test_case)
                self.assertEqual(test_case.test_success, True, test_case.fail_msg)
        finally:
            playground.debug = False
            playground.cleanup()



# Copyright 2012 Gentoo Foundation
# Distributed under the terms of the GNU General Public License v2

from portage import os, _supported_eapis
from portage.dep import use_reduce
from portage.eapi import eapi_has_automatic_unpack_dependencies
from portage.exception import InvalidDependString
from portage.localization import _
from portage.util import grabfile, writemsg


def load_unpack_dependencies_configuration(repositories):
    repo_dict = {}
    for repo in repositories.repos_with_profiles():
        for eapi in _supported_eapis:
            if eapi_has_automatic_unpack_dependencies(eapi):
                file_name = os.path.join(
                    repo.location, "profiles", "unpack_dependencies", eapi
                )
                lines = grabfile(file_name, recursive=True)
                for line in lines:
                    elements = line.split()
                    suffix = elements[0].lower()
                    if len(elements) == 1:
                        writemsg(
                            _(
                                "--- Missing unpack dependencies for '%s' suffix in '%s'\n"
                            )
                            % (suffix, file_name)
                        )
                    depend = " ".join(elements[1:])
                    try:
                        use_reduce(depend, eapi=eapi)
                    except InvalidDependString as e:
                        writemsg(
                            _(
                                "--- Invalid unpack dependencies for '%s' suffix in '%s': '%s'\n"
                                % (suffix, file_name, e)
                            )
                        )
                    else:
                        repo_dict.setdefault(repo.name, {}).setdefault(eapi, {})[
                            suffix
                        ] = depend

    ret = {}
    for repo in repositories.repos_with_profiles():
        for repo_name in [x.name for x in repo.masters] + [repo.name]:
            for eapi in repo_dict.get(repo_name, {}):
                for suffix, depend in (
                    repo_dict.get(repo_name, {}).get(eapi, {}).items()
                ):
                    ret.setdefault(repo.name, {}).setdefault(eapi, {})[suffix] = depend

    return ret
